(defun compile-pkg (&optional command startdir)
  "Compile a package, moving up to the parent directory
  containing configure.ac, if it exists. Start in startdir if defined,
  else start in the current directory."
  (interactive)
  (let ((dirname)
        (dir-buffer nil))
    (setq startdir (expand-file-name (if startdir startdir ".")))
    (setq command  (if command command compile-command))

    (setq dirname (upward-find-files '("configure.ac" "configure.in")
                                     startdir))
    (setq dirname (if dirname dirname (upward-find-file "Makefile" startdir)))
    (setq dirname (if dirname dirname (expand-file-name ".")))
                                        ; We've now worked out where to start. Now we need to worry about
                                        ; calling compile in the right directory
    (save-excursion
      (setq dir-buffer (find-file-noselect dirname))
      (set-buffer dir-buffer)
      (compile command)
      (kill-buffer dir-buffer))))

(defun upward-find-files (filenames startdir)
	"Move up directories until we find one of a list of
  filenames. If we manage to find one, return the containing
  directory. Else if we get to the root directory and still
  can't find it, return nil. Start at startdir."
	(let ((dirname startdir)
				(found nil) ; found is set as a flag to leave loop if we find it
				(top nil)) ; top is set when we get to / so that we only check it once
                                        ; While we've neither been at the top last time nor have we found
                                        ; the file.
    (while (not (or found top))
                                        ; If we're at / set top flag.
      (if (string= (expand-file-name dirname) "/")
					(setq top t))
                                        ; Check for the files
			(dolist (name filenames)
				(if (file-exists-p (expand-file-name name dirname))
						(setq found t)))
			(if (not found)
					(setq dirname (expand-file-name ".." dirname))))

                                        ; return statement
    (if found dirname nil)))

(defun upward-find-file (filename &optional startdir)
  "Move up directories until we find a certain filename. If we
  manage to find it, return the containing directory. Else if we
  get to the toplevel directory and still can't find it, return
  nil. Start at startdir or . if startdir not given"
  (let ((dirname
				 (expand-file-name (if startdir startdir "."))))
		(upward-find-files (cons filename '()) dirname)))

(defvar my-compile-command "make"
  "The compile command used by std-compile to drive compile-pkg.")

(defun switch-compile (prefix-arg)
  "If compile-command = \"make\", switch to \"make install\" and
  vice versa. With a prefix, prompt for a new command."
  (interactive "P")
  
  (setf my-compile-command
        (cond
         (prefix-arg (read-from-minibuffer "New compile command: "))
         ((string= my-compile-command "make") "make install")
         (t "make")))
  (message (concat "Compile command: " my-compile-command)))

(defun make-tags ()
  "Make etags in source directory by running compile-pkg with
  make tags as the command"
  (interactive)
  (compile-pkg "make tags"))

(defun std-compile ()
  "Like 'compile', but uses compile-pkg"
  (interactive)
  (compile-pkg my-compile-command))
